import java.text.*;
import com.imsl.stat.*;

public class  KaplanMeierEstimatesEx1 {
    public static void main(String args[]) {
        
        double[][] x = {
            {143, 5, 0, 1}, {164, 5, 0, 1}, {188, 5, 0, 2}, {190, 5, 0, 1},
            {192, 5, 0, 1}, {206, 5, 0, 1}, {209, 5, 0, 1}, {213, 5, 0, 1},
            {216, 5, 0, 1}, {220, 5, 0, 1}, {227, 5, 0, 1}, {230, 5, 0, 1},
            {234, 5, 0, 1}, {246, 5, 0, 1}, {265, 5, 0, 1}, {304, 5, 0, 1},
            {216, 5, 1, 1}, {244, 5, 1, 1}, {142, 7, 0, 1}, {156, 7, 0, 1},
            {163, 7, 0, 1}, {198, 7, 0, 1}, {205, 7, 0, 1}, {232, 7, 0, 2},
            {233, 7, 0, 4}, {239, 7, 0, 1}, {240, 7, 0, 1}, {261, 7, 0, 1},
            {280, 7, 0, 2}, {296, 7, 0, 2}, {323, 7, 0, 1}, {204, 7, 1, 1},
            {344, 7, 1, 1}
        };
        int nobs = x.length, censorIndex = 2, frequencyIndex = 3;
        int stratumIndex = 1, responseIndex = 0;
        int i, totalFailing, groupTotal, groupValue;
        int atRisk[] = new int[nobs];
        int nFailing[] = new int[nobs];
        double prob[] = new double[nobs];
        double se[] = new double[nobs];
        double logLikelihood;
        
        // Get Kaplan-Meir Estimates
        
        KaplanMeierEstimates km = new KaplanMeierEstimates(x);
        km.setCensorColumn(censorIndex);
        km.setFrequencyColumn(frequencyIndex);
        km.setStratumColumn(stratumIndex);
        atRisk = km.getNumberAtRisk();
        nFailing = km.getNumberOfFailures();
        prob = km.getSurvivalProbabilities();
        se = km.getStandardErrors();
        
        // Print Results
        NumberFormat nf = NumberFormat.getInstance();
        nf.setMaximumFractionDigits(4);
        nf.setMinimumFractionDigits(4);
        i = 0;
        while(i < nobs){
            groupValue = (int) x[i][stratumIndex];
            System.out.println("\n       Kaplan-Meier Survival Probabilities");
            System.out.println("               For Group Value = "+groupValue);
            System.out.println("\nNumber     Number"
                    +"                Survival     Estimated");
            System.out.println("at risk    Failing     Time"
                    +"     Probability   Std. Error");
            while(i < nobs && ((int)x[i][stratumIndex] == groupValue)){
                if((int)x[i][censorIndex] != 1){
                    System.out.println("  "+atRisk[i]+"          "+nFailing[i]+"" +
                            "         "+((int)x[i][responseIndex])+"       "
                            +nf.format(prob[i])+"        "+nf.format(se[i]));
                }
                i ++;
            }
            System.out.println("\nTotal number in group = "+
                    km.getGroupTotal(groupValue));
            System.out.println("Total number failing = "+
                    km.getTotalNumberOfFailures(groupValue));
            System.out.println("Product Limit likelihood = "+
                    nf.format(km.getLogLikelihood(groupValue)));
        }
        System.out.println("\n\n\nThe number of rows of x with missing values is "+
                km.getNumberOfRowsMissing());
    }
}
